@@ -213,8 +213,9 @@ class Config:
             "logging": {
                 "debug": False,
                 "debugmodes": [0, 1],
+                "debuglogsdir": os.path.join(LOGDIR, "debug"),
                 "logcollapsed": False,
-                "logsdir": LOGDIR,
+                "transferslogsdir": os.path.join(LOGDIR, "transfers"),
                 "rooms_timestamp": "%H:%M:%S",
                 "private_timestamp": "%Y-%m-%d %H:%M:%S",
                 "log_timestamp": "%Y-%m-%d %H:%M:%S",
@@ -222,6 +223,7 @@ class Config:
                 "privatechat": False,
                 "chatrooms": False,
                 "transfers": False,
+                "debug_file_output": False,
                 "roomlogsdir": os.path.join(LOGDIR, "rooms"),
                 "privatelogsdir": os.path.join(LOGDIR, "private"),
                 "readroomlogs": True,
@@ -275,6 +277,7 @@ class Config:
             },
 
             "ui": {
+                "dark_mode": False,
                 "icontheme": "",
                 "chatme": "FOREST GREEN",
                 "chatremote": "",
@@ -354,7 +357,7 @@ class Config:
                 "height": 600,
                 "xposition": -1,
                 "yposition": -1,
-                "maximized": "True",
+                "maximized": True,
                 "urgencyhint": True
             },
 
@@ -587,13 +590,17 @@ class Config:
         # Remove everything ticker-related, no longer necessary after the introduction of room walls
         self.removeOldSection("ticker")
 
+        # Remove old log folder option, superseded by individual log folders for transfers and debug messages
+        self.removeOldOption("logging", "logsdir")
+
         # Checking for unknown section/options
         unknown1 = [
             'login', 'passw', 'enc', 'downloaddir', 'uploaddir', 'customban',
-            'descr', 'pic', 'logsdir', 'roomlogsdir', 'privatelogsdir',
+            'descr', 'pic', 'transferslogsdir', 'roomlogsdir', 'privatelogsdir',
             'incompletedir', 'autoreply', 'afterfinish', 'downloadregexp',
             'afterfolder', 'default', 'chatfont', 'npothercommand', 'npplayer',
-            'npformat', 'private_timestamp', 'rooms_timestamp', 'log_timestamp'
+            'npformat', 'private_timestamp', 'rooms_timestamp', 'log_timestamp',
+            'debuglogsdir'
         ]
 
         unknown2 = {
@@ -605,7 +612,8 @@ class Config:
                 "chat_hidebuttons", "tab_status_icons", "notexists",
                 "speechenabled", "enablefilters", "width",
                 "height", "xposition", "yposition", "labelmain", "labelrooms",
-                "labelprivate", "labelinfo", "labelbrowse", "labelsearch"
+                "labelprivate", "labelinfo", "labelbrowse", "labelsearch", "maximized",
+                "dark_mode"
             ],
             'words': [
                 "completion", "censorwords", "replacewords", "autoreplaced",
