@@ -48,14 +48,14 @@ from pynicotine.gtkgui.utils import InitialiseColumns
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import PressHeader
 from pynicotine.gtkgui.utils import ScrollBottom
-from pynicotine.gtkgui.utils import WriteLog
+from pynicotine.gtkgui.utils import TextSearchBar
 from pynicotine.gtkgui.utils import expand_alias
 from pynicotine.gtkgui.utils import is_alias
 from pynicotine.gtkgui.utils import showCountryTooltip
 from pynicotine.logfacility import log
 from pynicotine.utils import CleanFile
 from pynicotine.utils import cmp
-from pynicotine.utils import debug
+from pynicotine.utils import write_log
 
 
 def GetCompletion(part, list):
@@ -95,7 +95,7 @@ class RoomsControl:
         self.PrivateRooms = config["private_rooms"]["rooms"]
 
         # Config cleanup
-        for room, data in list(self.PrivateRooms.items()):
+        for room, data in self.PrivateRooms.items():
             if "owner" not in data:
                 self.PrivateRooms[room]["owner"] = None
             if "operator" in data:
@@ -203,7 +203,7 @@ class RoomsControl:
         room_tab_order = {}
 
         # Find position of opened autojoined rooms
-        for name, room in list(self.joinedrooms.items()):
+        for name, room in self.joinedrooms.items():
 
             if name not in self.frame.np.config.sections["server"]["autojoin"]:
                 continue
@@ -235,7 +235,7 @@ class RoomsControl:
 
         page = notebook.get_nth_page(page_num)
 
-        for name, room in list(self.joinedrooms.items()):
+        for name, room in self.joinedrooms.items():
             if room.Main == page:
                 GLib.idle_add(room.ChatEntry.grab_focus)
 
@@ -249,7 +249,7 @@ class RoomsControl:
 
         page = self.ChatNotebook.get_nth_page(self.ChatNotebook.get_current_page())
 
-        for name, room in list(self.joinedrooms.items()):
+        for name, room in self.joinedrooms.items():
             if room.Main == page:
                 # Remove hilite
                 self.frame.Notifications.Clear("rooms", None, name)
@@ -259,7 +259,7 @@ class RoomsControl:
         if not focused:
             return
 
-        for name, room in list(self.users.items()):
+        for name, room in self.users.items():
             if room.Main == page:
                 self.frame.Notifications.Clear("rooms", name)
 
@@ -339,7 +339,6 @@ class RoomsControl:
             angle = int(self.frame.np.config.sections["ui"]["labelrooms"])
         except Exception as e:
             print(e)
-            pass
 
         self.ChatNotebook.append_page(room.Main, 'Public ', room.OnLeave, angle)
 
@@ -395,7 +394,7 @@ class RoomsControl:
         try:
             angle = int(self.frame.np.config.sections["ui"]["labelrooms"])
         except Exception as e:
-            debug(e)
+            print(e)
 
         self.ChatNotebook.append_page(tab.Main, msg.room, tab.OnLeave, angle)
 
@@ -414,7 +413,9 @@ class RoomsControl:
                 room_list = self.frame.np.config.sections["server"]["autojoin"]
 
             for room in room_list:
-                if room[-1:] != ' ':
+                if room == 'Public ':
+                    self.OnJoinPublicRoom(None)
+                else:
                     self.frame.np.queue.put(slskmessages.JoinRoom(room))
 
         self.roomsmodel.clear()
@@ -614,15 +615,15 @@ class RoomsControl:
                 self.PrivateRooms[msg.room]["owner"] = None
 
     def GetUserStats(self, msg):
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.GetUserStats(msg.user, msg.avgspeed, msg.files)
 
     def GetUserStatus(self, msg):
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.GetUserStatus(msg.user, msg.status)
 
     def SetUserFlag(self, user, flag):
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.SetUserFlag(user, flag)
 
     def GetUserAddress(self, user):
@@ -676,7 +677,7 @@ class RoomsControl:
 
         self.frame.SetTextBG(self.frame.roomlist.SearchRooms)
 
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.ChangeColours()
 
     def saveColumns(self):
@@ -689,7 +690,7 @@ class RoomsControl:
             if room not in self.joinedrooms:
                 del self.frame.np.config.sections["columns"]["chatrooms_widths"][room]
 
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.saveColumns()
 
     def LeaveRoom(self, msg):
@@ -709,7 +710,7 @@ class RoomsControl:
 
         self.roomsmodel.clear()
 
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.ConnClose()
 
         self.autojoin = 1
@@ -738,7 +739,7 @@ class RoomsControl:
 
             self.clist = clist
 
-        for room in list(self.joinedrooms.values()):
+        for room in self.joinedrooms.values():
             room.GetCompletionList(clist=list(self.clist))
 
 
@@ -781,6 +782,13 @@ class ChatRoom:
 
         self.clist = []
 
+        # Log Text Search
+        TextSearchBar(self.RoomLog, self.LogSearchBar, self.LogSearchEntry)
+
+        # Chat Text Search
+        TextSearchBar(self.ChatScroll, self.ChatSearchBar, self.ChatSearchEntry)
+
+        # Spell Check
         if self.frame.gspell and self.frame.np.config.sections["ui"]["spellcheck"]:
             from gi.repository import Gspell
             spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(self.ChatEntry.get_buffer())
@@ -853,7 +861,7 @@ class ChatRoom:
             gobject.TYPE_STRING
         )
 
-        for (username, user) in users.items():
+        for username, user in users.items():
 
             img = self.frame.GetStatusImage(user.status)
             flag = user.country
@@ -1015,24 +1023,11 @@ class ChatRoom:
 
         GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
 
-    def on_key_press_event(self, widget, event):
-
-        key = Gdk.keyval_name(event.keyval)
-
-        # Match against capslock + control and control
-        if key in ("f", "F") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
-            self.OnFind(widget)
-        elif key in ("F3"):
-            self.OnFind(widget, repeat=True)
-
-    def OnFind(self, widget, repeat=False):
-        self.frame.OnFindTextview(None, widget, repeat=repeat)
-
     def OnFindLogWindow(self, widget):
-        self.frame.OnFindTextview(None, self.RoomLog)
+        self.LogSearchBar.set_search_mode(True)
 
     def OnFindChatLog(self, widget):
-        self.frame.OnFindTextview(None, self.ChatScroll)
+        self.ChatSearchBar.set_search_mode(True)
 
     def destroy(self):
         self.Main.destroy()
@@ -1146,7 +1141,7 @@ class ChatRoom:
     def TickerSet(self, msg):
 
         self.Tickers.set_ticker([])
-        for user in list(msg.msgs.keys()):
+        for user in msg.msgs:
             if user in self.frame.np.config.sections["server"]["ignorelist"] or self.frame.UserIpIsIgnored(user):
                 # User ignored, ignore Ticker messages
                 return
@@ -1231,7 +1226,8 @@ class ChatRoom:
 
         line = "\n-- ".join(line.split("\n"))
         if self.Log.get_active():
-            WriteLog(self.frame.np.config.sections["logging"]["roomlogsdir"], self.room, line)
+            timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
+            write_log(self.frame.np.config.sections["logging"]["roomlogsdir"], self.room, line, timestamp_format)
 
         self.getUserTag(user)
 
@@ -1505,7 +1501,9 @@ class ChatRoom:
         hspeed = HumanSpeed(userdata.avgspeed)
         hfiles = Humanize(userdata.files)
 
-        self.users[username] = self.usersmodel.append([img, self.frame.GetFlagImage(flag), username, hspeed, hfiles, userdata.status, userdata.avgspeed, userdata.files, flag])
+        self.users[username] = self.usersmodel.append(
+            [img, self.frame.GetFlagImage(flag), username, hspeed, hfiles, userdata.status, userdata.avgspeed, userdata.files, flag]
+        )
 
         self.getUserTag(username)
 
@@ -1565,9 +1563,9 @@ class ChatRoom:
 
     def UserColumnDraw(self, column, cellrenderer, model, iter, dummy="dummy"):
 
-        user = self.usersmodel.get_value(iter, 2)
-
         if self.room in self.roomsctrl.PrivateRooms:
+            user = self.usersmodel.get_value(iter, 2)
+
             if user == self.roomsctrl.PrivateRooms[self.room]["owner"]:
                 cellrenderer.set_property("underline", pango.Underline.SINGLE)
                 cellrenderer.set_property("weight", pango.Weight.BOLD)
@@ -1577,9 +1575,6 @@ class ChatRoom:
             else:
                 cellrenderer.set_property("weight", pango.Weight.NORMAL)
                 cellrenderer.set_property("underline", pango.Underline.NONE)
-        else:
-            cellrenderer.set_property("weight", pango.Weight.NORMAL)
-            cellrenderer.set_property("underline", pango.Underline.NONE)
 
         self.frame.CellDataFunc(column, cellrenderer, model, iter)
 
@@ -1816,7 +1811,7 @@ class ChatRoom:
         if not self.AutoJoin.get_active() and self.room in config["columns"]["chatrooms_widths"]:
             del config["columns"]["chatrooms_widths"][self.room]
 
-        for tag in list(self.tag_users.values()):
+        for tag in self.tag_users.values():
             self.changecolour(tag, "useroffline")
 
         self.Tickers.set_ticker([])
@@ -2110,7 +2105,7 @@ class ChatRooms(IconNotebook):
 
             n = self.page_num(child)
             page = self.get_nth_page(n)
-            room = [room for room, tab in list(self.roomsctrl.joinedrooms.items()) if tab.Main is page][0]
+            room = [room for room, tab in self.roomsctrl.joinedrooms.items() if tab.Main is page][0]
 
             if event.button == 2:
                 self.roomsctrl.joinedrooms[room].OnLeave(widget)
