@@ -39,12 +39,13 @@ from pynicotine.gtkgui.utils import AppendLine
 from pynicotine.gtkgui.utils import IconNotebook
 from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import ScrollBottom
-from pynicotine.gtkgui.utils import WriteLog
+from pynicotine.gtkgui.utils import TextSearchBar
 from pynicotine.gtkgui.utils import expand_alias
 from pynicotine.gtkgui.utils import is_alias
 from pynicotine.logfacility import log
 from pynicotine.utils import CleanFile
 from pynicotine.utils import version
+from pynicotine.utils import write_log
 
 
 CTCP_VERSION = "\x01VERSION\x01"
@@ -359,6 +360,10 @@ class PrivateChat:
         self.status = -1
         self.clist = []
 
+        # Text Search
+        TextSearchBar(self.ChatScroll, self.SearchBar, self.SearchEntry)
+
+        # Spell Check
         if self.frame.gspell and self.frame.np.config.sections["ui"]["spellcheck"]:
             from gi.repository import Gspell
             spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(self.ChatLine.get_buffer())
@@ -488,7 +493,7 @@ class PrivateChat:
         return True
 
     def OnFindChatLog(self, widget):
-        self.frame.OnFindTextview(widget, self.ChatScroll)
+        self.SearchBar.set_search_mode(True)
 
     def OnCopyChatLog(self, widget):
 
@@ -548,7 +553,8 @@ class PrivateChat:
             AppendLine(self.ChatScroll, line, tag, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
 
         if self.Log.get_active():
-            WriteLog(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line)
+            timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
+            write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
         autoreply = self.frame.np.config.sections["server"]["autoreply"]
         if self.frame.away and not self.autoreplied and autoreply:
@@ -590,7 +596,8 @@ class PrivateChat:
         AppendLine(self.ChatScroll, line, tag, timestamp_format=timestamp_format, username=my_username, usertag=usertag)
 
         if self.Log.get_active():
-            WriteLog(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line)
+            timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
+            write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
         if bytestring:
             payload = text
