@@ -22,7 +22,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os
 import re
 import sys
 import time
@@ -40,7 +39,6 @@ from gi.repository import Pango as pango
 from pynicotine import slskmessages
 from pynicotine.gtkgui.dialogs import EntryDialog
 from pynicotine.gtkgui.countrycodes import code2name
-from pynicotine.utils import CleanFile
 from pynicotine.utils import executeCommand
 
 
@@ -376,8 +374,11 @@ def OpenUri(uri, window):
             PROTOCOL_HANDLERS[protocol](uri.strip())
             return
         if PROTOCOL_HANDLERS[protocol]:
-            executeCommand(PROTOCOL_HANDLERS[protocol], uri)
-            return
+            try:
+                executeCommand(PROTOCOL_HANDLERS[protocol], uri)
+                return
+            except RuntimeError as e:
+                print(e)
 
     # Situation 2, user did not define a way of handling the protocol
     if sys.platform == "win32" and webbrowser:
@@ -1207,21 +1208,93 @@ class PopupMenu(gtk.Menu):
         return True
 
 
-def WriteLog(logsdir, fn, msg):
+class TextSearchBar:
 
-    oldumask = os.umask(0o077)
-    if not os.path.exists(logsdir):
-        os.makedirs(logsdir)
+    def __init__(self, textview, search_bar, entry):
 
-    logfile = open(os.path.join(logsdir, CleanFile(fn.replace(os.sep, "-")) + ".log"), 'ab', 0)
+        self.textview = textview
+        self.search_bar = search_bar
+        self.entry = entry
 
-    os.umask(oldumask)
+        self.search_bar.connect_entry(self.entry)
 
-    text = "%s %s\n" % (time.strftime(NICOTINE.np.config.sections["logging"]["log_timestamp"]), msg)
+        self.entry.connect("activate", self.OnSearchNextMatch)
+        self.entry.connect("search-changed", self.OnSearchChanged)
 
-    logfile.write(text.encode('UTF-8', 'replace'))
-    logfile.flush()
-    logfile.close()
+        self.entry.connect("previous-match", self.OnSearchPreviousMatch)
+        self.entry.connect("next-match", self.OnSearchNextMatch)
+
+        self.textview.connect("key-press-event", self.OnKeyPress)
+
+    def OnSearchChanged(self, widget):
+
+        buffer = self.textview.get_buffer()
+        start, end = buffer.get_bounds()
+        query = self.entry.get_text()
+
+        self.textview.emit("select-all", False)
+
+        iter = start
+        match = iter.forward_search(query, gtk.TextSearchFlags.TEXT_ONLY | gtk.TextSearchFlags.CASE_INSENSITIVE, limit=None)
+
+        if match is not None and len(match) == 2:
+            match_start, match_end = match
+            buffer.place_cursor(match_end)
+            buffer.select_range(match_end, match_start)
+            self.textview.scroll_to_iter(match_start, 0, False, 0.5, 0.5)
+
+    def OnSearchPreviousMatch(self, widget):
+
+        buffer = self.textview.get_buffer()
+        query = self.entry.get_text()
+
+        self.textview.emit("select-all", False)
+
+        current = buffer.get_mark("insert")
+        iter = buffer.get_iter_at_mark(current)
+        match = iter.backward_search(query, gtk.TextSearchFlags.TEXT_ONLY | gtk.TextSearchFlags.CASE_INSENSITIVE, limit=None)
+
+        if match is not None and len(match) == 2:
+            match_start, match_end = match
+            buffer.place_cursor(match_start)
+            buffer.select_range(match_start, match_end)
+            self.textview.scroll_to_iter(match_start, 0, False, 0.5, 0.5)
+        else:
+            start, end = buffer.get_bounds()
+            buffer.place_cursor(end)
+            self.OnSearchPreviousMatch(widget)
+
+    def OnSearchNextMatch(self, widget):
+
+        buffer = self.textview.get_buffer()
+        query = self.entry.get_text()
+
+        self.textview.emit("select-all", False)
+
+        current = buffer.get_mark("insert")
+        iter = buffer.get_iter_at_mark(current)
+        match = iter.forward_search(query, gtk.TextSearchFlags.TEXT_ONLY | gtk.TextSearchFlags.CASE_INSENSITIVE, limit=None)
+
+        if match is not None and len(match) == 2:
+            match_start, match_end = match
+            buffer.place_cursor(match_end)
+            buffer.select_range(match_end, match_start)
+            self.textview.scroll_to_iter(match_start, 0, False, 0.5, 0.5)
+        else:
+            start, end = buffer.get_bounds()
+            buffer.place_cursor(start)
+            self.OnSearchNextMatch(widget)
+
+    def OnKeyPress(self, widget, event):
+        key = Gdk.keyval_name(event.keyval)
+
+        # Match against capslock + control and control
+        if key in ("f", "F") and event.state in (Gdk.ModifierType.CONTROL_MASK, Gdk.ModifierType.LOCK_MASK | Gdk.ModifierType.CONTROL_MASK):
+            self.ShowSearchBar()
+
+    def ShowSearchBar(self):
+        self.search_bar.set_search_mode(True)
+        self.entry.grab_focus_without_selecting()
 
 
 size_suffixes = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB']
@@ -1245,7 +1318,7 @@ speed_suffixes = ['B/s', 'KiB/s', 'MiB/s', 'GiB/s', 'TiB/s', 'PiB/s', 'EiB/s', '
 
 def HumanSpeed(filesize):
     try:
-        step_unit = 1000.0
+        step_unit = 1024.0
 
         for i in speed_suffixes:
             if filesize < step_unit:
