@@ -29,12 +29,14 @@ import gettext
 import locale
 import os
 import sys
+import time
 
 from codecs import encode, decode
+from gettext import gettext as _
 from subprocess import PIPE
 from subprocess import Popen
 
-version = "2.1.0"
+version = "2.1.1"
 
 win32 = sys.platform.startswith("win")
 
@@ -92,31 +94,22 @@ def get_latest_version():
     latest = int(make_version(hlatest))
     date = data['created_at']
 
-    return latest, date
+    return hlatest, latest, date
 
 
 def make_version(version):
 
-    if version.find("dev") >= 0:
-        # Example: 2.0.1.dev1
-
-        ix = version.find("dev") - 1
-        version = version[:ix]
-    elif version.find("rc") >= 0:
-        # Example: 2.0.1.rc1
-
-        ix = version.find("rc") - 1
-        version = version[:ix]
-
     s = version.split(".")
+    major, minor, patch = (int(i) for i in s[:3])
+    stable = 1
 
-    if len(s) >= 4:
-        major, minor, micro, milli = [int(i) for i in s[:4]]
-    else:
-        major, minor, micro = [int(i) for i in s[:3]]
-        milli = 0
+    if "dev" in version or \
+            "rc" in version:
+        # Example: 2.0.1.dev1
+        # A dev version will be one less than a stable version
+        stable = 0
 
-    return (major << 24) + (minor << 16) + (micro << 8) + milli
+    return (major << 24) + (minor << 16) + (patch << 8) + stable
 
 
 def GetUserDirectories():
@@ -435,8 +428,40 @@ def executeCommand(command, replacement=None, background=True, returnoutput=Fals
     return procs[-1].communicate()[0]
 
 
+def cmp(a, b):
+    """Replacement for cmp() which is removed in Python 3"""
+    return (a > b) - (a < b)
+
+
+def write_log(logsdir, fn, msg, timestamp_format="%Y-%m-%d %H:%M:%S"):
+
+    oldumask = os.umask(0o077)
+    if not os.path.exists(logsdir):
+        os.makedirs(logsdir)
+
+    try:
+        with open(os.path.join(logsdir, CleanFile(fn.replace(os.sep, "-")) + ".log"), 'ab', 0) as logfile:
+            os.umask(oldumask)
+
+            text = "%s %s\n" % (time.strftime(timestamp_format), msg)
+            logfile.write(text.encode('UTF-8', 'replace'))
+
+    except IOError as error:
+        print(_("Couldn't write to log file \"%s\": %s") % (fn, error))
+
+
+""" Debugging """
+
+
+def debug(*args):
+    """ Prints debugging info. """
+
+    truncated_args = [arg[:200] if isinstance(arg, str) else arg for arg in args]
+    print('*' * 8, truncated_args)
+
+
 def strace(function):
-    """Decorator for debugging"""
+    """ Decorator for debugging """
 
     from itertools import chain
 
@@ -448,17 +473,3 @@ def strace(function):
         return retvalue
 
     return newfunc
-
-
-def cmp(a, b):
-    """Replacement for cmp() which is removed in Python 3"""
-    return (a > b) - (a < b)
-
-
-def debug(*args):
-    """
-    Prints debugging info.
-    TODO: add CLI switch --debug for en-/disabling.
-    """
-    truncated_args = [arg[:200] if isinstance(arg, str) else arg for arg in args]
-    print('*' * 8, truncated_args)
